/* dijkstra.c
 * Implementação do algoritmo de Dijkstra
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <stdbool.h>

#define BRANCO 0
#define CINZA  1
#define PRETO  2
#define MAX_VERTICES 100 // número máximo de vértices do grafo
#define INF INT_MAX // infinito

typedef struct {
	int dist, pedagio;
} aresta;

void dijkstra(aresta [][MAX_VERTICES], int, int, int, bool); // implementa o algortimo Dijkstra
void print_graph(int *, int, int); // exibe o caminho do vértice origem ao destino
void init_matriz(aresta [][MAX_VERTICES], int, int); // inidializa a matriz de adjacências

int main() {
  	aresta adj[MAX_VERTICES][MAX_VERTICES]; // matriz de adjacências
  	int i, c, e, x, y, n, o, d;
  	aresta peso;

	scanf("%d %d", &c, &e);
    init_matriz(adj, c, c);
    for (i = 1; i <= e; i++) {
      	scanf("%d %d %d %d", &x, &y, &peso.dist, &peso.pedagio);
      	adj[x][y] = peso;
      	adj[y][x] = peso; // Se o grafo não for orientado, descomentar
    }
    	
	scanf("%d", &n);
	for (i = 1; i <= n; i++) {
   		scanf("%d %d", &o, &d);
   		dijkstra(adj, c, o, d, true);
   		dijkstra(adj, c, o, d, false);
   	}
   	
  	return 0;
}

void init_matriz(aresta a[][MAX_VERTICES], int l, int c) {
	int i, j;
  	for (i = 0; i < l; i++){
    	for (j = 0; j < c; j++) {
      		a[i][j].pedagio = a[i][j]. dist = INF;
    	}
  	}
}

void dijkstra(aresta adj[][MAX_VERTICES], int v, int s, int destino, bool ped) {
  	static int cor[MAX_VERTICES], pai[MAX_VERTICES];
  	static aresta d[MAX_VERTICES];
  	int u, i, j, dist, peso;

  	for (u = 0; u < v; u++) {
    	cor[u] = BRANCO;
    	d[u].dist = d[u].pedagio = INF;
    	pai[u] = -1;    
  	}
  	cor[s] = CINZA;
  	d[s].dist = d[s].pedagio = 0;

  	for (i = 1; i <= v; i++) {
    	// menor distância
    	u = 0;
    	dist = INF;
    	for (j = 0; j < v; j++) {
    		peso = ped ? d[j].pedagio : d[j].dist;
      		if (cor[j] != PRETO && peso < dist) {
				u = j;
				dist = peso;
      		}
    	}

    	for (j = 0; j < v; j++) {
    		peso = ped ? adj[u][j].pedagio : adj[u][j].dist;
      		if (peso < INF) {
        		// relaxar
        		if (ped) {
        			if (d[j].pedagio > d[u].pedagio + adj[u][j].pedagio) {
	  					d[j].pedagio = d[u].pedagio + adj[u][j].pedagio;
	  					d[j].dist = d[u].dist + adj[u][j].dist;
	  					pai[j] = u;
					}
				}
				else {
					if (d[j].dist > d[u].dist + adj[u][j].dist) {
  						d[j].pedagio = d[u].pedagio + adj[u][j].pedagio;
  						d[j].dist = d[u].dist + adj[u][j].dist;
  						pai[j] = u;
					}
				}

				if (cor[j] == BRANCO) {
	  				cor[j] = CINZA;
				}
      		}
    	}
    	cor[u] = PRETO;
  	}

  	if (d[destino].pedagio != INF){
  		printf("%d %d\n", d[destino].dist, d[destino].pedagio);
    	print_graph(pai,s,destino); /* exibe o caminho de s a destino */
    	printf("\n");
  	}
  	else {
  		printf("Nenhum caminho de %d a %d\n", s, destino);
  	}
  	
}

void print_graph(int * pai, int s, int v) {
  	if (s == v) {
    	printf("%d", s);
  	}
  	else if (pai[v] == -1) {
    	printf("Nenhum caminho de %d para %d\n", s, v);
  	}
  	else {
    	print_graph(pai, s, pai[v]);
    	printf(" %d", v);
  	}
}
